
;*****************************************************************************
	
	LIST      P=16F873            ; LIST DIRECTIVE TO DEFINE PROCESSOR
	#INCLUDE <P16F873.INC>        ; PROCESSOR SPECIFIC VARIABLE DEFINITIONS
	ERRORLEVEL      -302   		; SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -306   		; SUPPRESS BANK SELECTION MESSAGES	


        __CONFIG  _CP_ALL & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _HS_OSC & _LVP_OFF & _DEBUG_OFF & _CPD_OFF

;*****************************************************************************
;  REGISTER USAGE

  CBLOCK H'20'                   ;  START REGISTERS AT END OF THE VALUES
HOUR_H
HOUR_L
MIN_H
MIN_L
SEC_H
SEC_L

FLAG 
BUFW
ADRBUF
DATABUF
SER_BUF

D1
D2
D3
 ENDC

PUSH		EQU	0	
KEYS		EQU	1
IOUT		EQU	2
UPDATE		EQU	3
CL_REDRAW	EQU	4
T_REDRAW	EQU	5
;
;*****************************************************************************

#DEFINE		DS	PORTB,1
#DEFINE		RW	PORTB,2
#DEFINE		AS	PORTB,4

;  MACROS

BANK0	MACRO
	BCF	STATUS,RP0
	BCF	STATUS,RP1
	ENDM
	
BANK1	MACRO
	BSF	STATUS,RP0
	BCF	STATUS,RP1
	ENDM
		
;================================================================
; START
;================================================================
	ORG	H'0'			; RESET VECTOR LOCATION
;P0
	GOTO	START
;*****************************************************************************
; INTERRUPT
;*****************************************************************************
	ORG	H'4'			; INTERRUPT VECTOR LOCATION
	GOTO    START
;====================================================================
	ORG	H'6'
;================================================

;================================================	
START	
;	CALL	DELAY1S
	BANK1
	MOVLW	H'07'
	MOVWF	ADCON1		; AN0-AN7 - DIGITAL INPUTS
	BANK0
;==========================================================
	CLRF	PORTC

	MOVLW	B'00000000'
	MOVWF	PORTB
;==================== PORT A ==============================
	BANK1
	MOVLW	B'00011111'    	; 
	MOVWF	TRISA 		; 
;==================== PORT B ==============================
	MOVLW	B'00000000'	;
	MOVWF	TRISB		;
;==================== PORT C ==============================	
	MOVLW	B'11111111'	;
	MOVWF	TRISC
;==========================================================
	MOVLW	B'0001111'	;
	MOVWF	OPTION_REG
	BANK0
	CLRF	INTCON
;==========================================================	
	CLRF	PORTC
	CLRF	PORTB
;==========================================================

;==========================================================
	MOVLW	H'20'
	MOVWF	FSR
CL
	CLRF	INDF
	INCF	FSR,F
	BTFSS	FSR,7		
	GOTO	CL

	BANK0
	
;================================================
; INIT DS12887
;================================================
;REG B
	MOVLW	B'00000010'
	MOVWF	DATABUF
	MOVLW	D'11'
	MOVWF	ADRBUF
	CALL	DS_WRITE
	
;REG A
	MOVLW	B'00100010'
	MOVWF	DATABUF

	MOVLW	D'10'
	MOVWF	ADRBUF
	CALL	DS_WRITE
	
;REG C
	MOVLW	B'00000000'
	MOVWF	DATABUF
	MOVLW	D'12'
	MOVWF	ADRBUF
	CALL	DS_WRITE

;================================================
PROGRAMM
	CALL	READ_CLOCK
;	CALL	DELAY1S
	GOTO	PROGRAMM

;=====================================================
; 
;================================================
WRITE_CLOCK
;
	MOVLW	B'10000011'
	MOVWF	DATABUF
	MOVLW	D'11'
	MOVWF	ADRBUF
	CALL	DS_WRITE

;
	MOVLW	D'4'
	MOVWF	ADRBUF
	SWAPF	HOUR_H,W	;
	ADDWF	HOUR_L,W	
	MOVWF	DATABUF
	CALL	DS_WRITE

;
	MOVLW	D'2'
	MOVWF	ADRBUF
	SWAPF	MIN_H,W		;
	ADDWF	MIN_L,W
	MOVWF	DATABUF
	CALL	DS_WRITE

;
	CLRF	ADRBUF
	CLRF	DATABUF
	CALL	DS_WRITE

;
	MOVLW	B'00000011'
	MOVWF	DATABUF
	MOVLW	D'11'
	MOVWF	ADRBUF
	CALL	DS_WRITE

	RETURN
;================================================
; 
;================================================
READ_CLOCK
;

	MOVLW	D'4'
	MOVWF	ADRBUF
	CALL	DS_READ
	SWAPF	DATABUF,W
	ANDLW	B'00001111'
	MOVWF	HOUR_H
	MOVFW	DATABUF
	ANDLW	B'00001111'
	MOVWF	HOUR_L		

;

	MOVLW	D'2'
	MOVWF	ADRBUF
	CALL	DS_READ
	SWAPF	DATABUF,W
	ANDLW	B'00001111'
	MOVWF	MIN_H
	MOVFW	DATABUF
	ANDLW	B'00001111'
	MOVWF	MIN_L	
	
; 
	
	MOVLW	D'0'
	MOVWF	ADRBUF
	CALL	DS_READ
	SWAPF	DATABUF,W
	ANDLW	B'00001111'
	MOVWF	SEC_H
	MOVFW	DATABUF
	ANDLW	B'00001111'
	MOVWF	SEC_L
	RETURN
;================================================

;================================================
DS_WRITE
; 	
	BANK1
	MOVLW	B'00000000'	;
	MOVWF	TRISC
	BANK0

	MOVFW	ADRBUF	; 
	MOVWF	PORTC	; 
;1	
	BCF	AS	;
	BSF	DS
	BSF	RW
;2
	BSF	AS
;4
	BCF	AS
	NOP
;5
	BCF	RW
;6
	MOVFW	DATABUF
	MOVWF	PORTC	; 
	NOP
;7	
	BSF	RW	; 
	NOP

; 
	BANK1
	MOVLW	B'11111111'	;
	MOVWF	TRISC
	BANK0	

	RETURN
;================================================

DS_READ
; 
	BANK1
	MOVLW	B'00000000'	;
	MOVWF	TRISC
	BANK0

	MOVFW	ADRBUF	; 
	MOVWF	PORTC	; 
;1	
	BCF	AS	;
	BSF	DS
	BSF	RW
;2
	BSF	AS
	NOP		;
;4
	BCF	AS
	NOP
;5
	BCF	DS
;============
; 
	BANK1
	MOVLW	B'11111111'	;
	MOVWF	TRISC
	BANK0	
;==================
;6	
	BSF	DS	; 
	NOP

	MOVFW	PORTC
	MOVWF	DATABUF
	NOP
;8 9
	BCF	DS
	BCF	RW

	RETURN

;================================================
;PIC TIME DELAY = 1.0000020 S WITH OSC = 4.000000 MHZ
DELAY1S
	CLRWDT
	MOVLW	D'6'
	MOVWF	D3
	MOVLW	D'24'
	MOVWF	D2
	MOVLW	D'168'
	MOVWF	D1
LOOP11	DECFSZ	D1,1
	GOTO	LOOP11
	DECFSZ	D2,1
	GOTO	LOOP11
	DECFSZ	D3,1
	GOTO	LOOP11
	RETURN	
	BANK0  
;=====================================================

;=====================================================

	END	